/*
 *
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * 5724-U18
 *
 * (C) COPYRIGHT IBM CORP. 2006,2014
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *
 */

package psdi.iface.action;

import javax.ejb.*;

import javax.xml.stream.*;

import java.io.*;
import java.lang.reflect.*;
import java.rmi.RemoteException;
import java.sql.Connection;
import java.util.*;

import psdi.iface.bidi.maximo.MaximoBidiTransformer;
import psdi.iface.mos.*;
import psdi.iface.query.MboQbeXMLProcessor;
import psdi.iface.util.*;
import psdi.util.*;
import psdi.iface.webservices.action.*;
import psdi.iface.webservices.*;
import psdi.iface.mic.*;
import psdi.mbo.*;

import org.jdom.*;

import com.ibm.tivoli.maximo.log.LogInfo;
import com.ibm.tivoli.maximo.log.MaxLoggerContext;
import com.ibm.tivoli.maximo.log.ThreadLoggerCache;

import psdi.security.ConnectionKey;
import psdi.security.UserInfo;
import psdi.server.*;
import psdi.txn.MXTransaction;
/**
 * EJB for invoking action services. 
 * <p>
 * 
 * @author Anamitra Bhattacharyya
 */

public class MAXActionServiceBean implements SessionBean
{
    private SessionContext ctx;
    private String ssNs;


    public void ejbActivate()
    {
    }

    public void ejbCreate()
    {
    }

    public void ejbRemove()
    {
        // closeAllProducers()
    }

    public void ejbPassivate()
    {
    }

    public void setSessionContext(SessionContext ctx)
    {
        this.ctx = ctx;
    }
    

    public byte[] action(byte[] actionData, String maxServiceName)
    {
        try
        {
            UserInfo userInfo = SecurityUtil.getUserInfo();
            //System.out.println("isAllowDefaultLogin"+isAllowDefaultLogin());
        	this.setUserToContext(userInfo);
            return secureActionInternal(userInfo,actionData,maxServiceName);
        }
        catch(MXException e)
        {
            throw new EJBException(e);
        }

    }
    
    public byte[] wsAction(byte[] actionData, String wsName)
    {
        try
        {
        	WSRegistryInfo info = WSRegistryCache.getInstance().getWSRegistryInfo(wsName);
            if(info == null)
            {
            	String[] params = {wsName};
            	throw new MXApplicationException("iface","unregisteredservice",params);
            }

            return action(actionData, info.getServiceName());
        }
        catch(MXException e)
        {
            throw new EJBException(e);
        }

    }

    
    public byte[] maxSecureAction(String loginid, String password, byte[] actionData, 
            String maxServiceName)
    {
    	UserInfo userInfo = null;
        try
        {
            userInfo = SecurityUtil.getUserInfo(loginid, password, (java.security.Principal)null);
            this.setUserToContext(userInfo);
            return secureActionInternal(userInfo,actionData,maxServiceName);
        }
        catch(MXException e)
        {
            throw new EJBException(e);
        }
        finally
        {
        	try
        	{
            	SecurityUtil.disconnectUser(userInfo);
        	}
        	catch(Exception e)
        	{
        		MicUtil.INTEGRATIONLOGGER.error(e.getMessage(), e);
        	}
        }
    }

    public byte[] wsMaxSecureAction(String loginid, String password, byte[] actionData, 
            String wsName)
    {
        try
        {
            WSRegistryInfo info = WSRegistryCache.getInstance().getWSRegistryInfo(wsName);
            if(info == null)
            {
            	String[] params = {wsName};
            	throw new MXApplicationException("iface","unregisteredservice",params);
            }

            return this.maxSecureAction(loginid, password, actionData, info.getServiceName());
        }
        catch(MXException e)
        {
            throw new EJBException(e);
        }
    }

    /**
     * Gateway method to MAXIMO. Processes the data for validity and writes the
     * data to the Queue - thus guarenteeing delivery. Will reject the data if
     * the data interface is disabled in MAXIMO. 
     * @return String The response code form MAXIMO.
     * 
     */
    public byte[] secureAction(byte[] actionData, 
            String maxServiceName)
    {
        try
        {
            UserInfo userInfo = SecurityUtil.getUserInfo(null, null, ctx.getCallerPrincipal());
            this.setUserToContext(userInfo);
            return secureActionInternal(userInfo,actionData,maxServiceName);
        }
        catch(MXException e)
        {
            throw new EJBException(e);
        }
    }
    
    
    public byte[] wsSecureAction(byte[] actionData, 
    		String wsName)
    {
        try
        {
            WSRegistryInfo info = WSRegistryCache.getInstance().getWSRegistryInfo(wsName);
            if(info == null)
            {
            	String[] params = {wsName};
            	throw new MXApplicationException("iface","unregisteredservice",params);
            }

            return secureAction(actionData,info.getServiceName());
        }
        catch(MXException e)
        {
            throw new EJBException(e);
        }
    }
    
    protected String getBidiFormat(String maxServiceName)
    {
    	try
    	{
    		String bidiFormat = null;
    		if(!((MaxPropCache)MXServer.getMXServer().getFromMaximoCache("MAXPROP")).
    				getProperties().containsKey("mxe.int.bidi.os."+maxServiceName.toLowerCase()))
    		{
    			bidiFormat = MXServer.getMXServer().getProperty("mxe.int.bidi.ss");
    		}
    		else
    		{
    			bidiFormat = MXServer.getMXServer().getProperty("mxe.int.bidi.ss."+maxServiceName.toLowerCase());
    		}
    		
    		return bidiFormat;

    	}
    	catch(RemoteException re)
    	{
    		MicUtil.INTEGRATIONLOGGER.error(re.getMessage(),re);
    		return null;
    	}
    }
    
    protected Document transformToMaximoBidiFormatFrom(Document doc, String bidiFormat) 
    	throws MXException, UnsupportedEncodingException
    {
    	if(bidiFormat != null && bidiFormat.trim().length() >0 && !bidiFormat.equalsIgnoreCase(MaximoBidiTransformer.MAXIMO_INBOUND_ILYNN))
    	{
    		doc = MaximoBidiTransformer.transformToMaximoFormat(doc, bidiFormat);
    		if(MicUtil.INTEGRATIONLOGGER.isDebugEnabled())
    		{
    			MicUtil.INTEGRATIONLOGGER.debug(new String(XMLUtils.convertDocumentToBytes(doc),"UTF-8"));
    		}
    		return doc;
    	}
    	return doc;
    }
    
    protected byte[] transformFromMaximoBidiFormatTo(byte[] responseData, String bidiFormat) 
		throws MXException, UnsupportedEncodingException
	{
		if(bidiFormat != null && bidiFormat.trim().length() >0 && !bidiFormat.equalsIgnoreCase(MaximoBidiTransformer.MAXIMO_OUTBOUND_ICYNN))
		{
			Document doc = XMLUtils.convertBytesToDocument(responseData);
			doc = MaximoBidiTransformer.transformFromMaximoFormat(doc, bidiFormat);
			if(MicUtil.INTEGRATIONLOGGER.isDebugEnabled())
			{
				MicUtil.INTEGRATIONLOGGER.debug(new String(XMLUtils.convertDocumentToBytes(doc),"UTF-8"));
			}
			return XMLUtils.convertDocumentToBytes(doc);
		}
		return responseData;
	}
    
    public Connection getDBConnection(ConnectionKey conKey) 
	{
    	try
    	{
    		return (MXServer.getMXServer().getDBManager().getConnection(conKey));
    	}
    	catch(RemoteException re)
    	{
    		MicUtil.INTEGRATIONLOGGER.error(re);
    		return null;
    	}
    }
    
    public void freeDBConnection(ConnectionKey conKey) 
	{    	
    	try
    	{
    		MXServer.getMXServer().getDBManager().freeConnection(conKey);
    	}
    	catch(RemoteException re)
    	{
    		MicUtil.INTEGRATIONLOGGER.error(re);
    	}

    }

    private void setThreadLogging(UserInfo userInfo)
    {
    	LogInfo logInfo = ThreadLoggerCache.getInstance().getLogInfo("STDMIF."+userInfo.getUserName());
    	if(logInfo != null)
    	{
    		MaxLoggerContext.createCurrentContext(logInfo, ""+psdi.iface.jms.MessageUtil.generateUniqueID(), userInfo);
    	}
    }


    private byte[] secureActionInternal(UserInfo userInfo, byte[] actionData, 
            String maxServiceName) throws MXException
    {
        MXTransaction mxTrans = null;
        Connection con = null;
        try
        {
            if (actionData == null)
            {
                throw new MXApplicationException("iface","nullreqxml");
            }
            this.setThreadLogging(userInfo);
            ssNs = MXServer.getMXServer().getProperty("mxe.int.xmlnamespace", Locale.getDefault().getLanguage());
            //userInfo = SecurityUtil.getUserInfo(userInfo, ctx);
            psdi.iface.webservices.action.ServiceInfo serviceInfo = MAXServiceDD.getServiceInfo(maxServiceName);
            MAXServiceSchemaGenerator schemaGen = new MAXServiceSchemaGenerator(maxServiceName);
            
            Map<String, MethodInfo> mapOfMethods = serviceInfo.getMethodInfoMap();
            Document doc = XMLUtils.convertBytesToDocument(actionData);
            
            String bidiFormat = this.getBidiFormat(maxServiceName);
            MicUtil.INTEGRATIONLOGGER.debug("standard services "+maxServiceName+" using the bidiFormat "+bidiFormat);
            doc = this.transformToMaximoBidiFormatFrom(doc, bidiFormat);

            Element rootElem = doc.getRootElement();
            
          	// issue # 08-16915 - throw error for root element namespace
           	String inNamespace = rootElem.getNamespaceURI();
           	if(! inNamespace.equals(ssNs))
           	{
           		String[] params = {inNamespace};
           		throw new MXApplicationException("iface", "invalidnamespace", params);
           	}
            	
            	
            String globalMethodName = rootElem.getName();
            MethodInfo methodInfo = mapOfMethods.get(globalMethodName);
            if(methodInfo == null)
            {
                String[] params = {maxServiceName,globalMethodName};
                throw new MXApplicationException("iface","invalidrootxmlelement",params);
            }
            
            String transLanguage = rootElem.getAttributeValue(MosConstants.TRANSLANGUAGE);
            if(transLanguage != null && transLanguage.trim().length()>0)
            {
                userInfo.setLangCode(transLanguage, MXServer.getMXServer());
            }
            Method serviceImplMethod = methodInfo.getMethod();
            AppService maxService = (AppService)MXServer.getMXServer().lookup(maxServiceName);
            
            ParamInfo[] paramsInfo = methodInfo.getParameters();
            Object[] paramVals = null;
            con = getDBConnection(userInfo.getConnectionKey());
            if(paramsInfo != null && paramsInfo.length>0)
            {
                mxTrans = MXServer.getMXServer().createMXTransaction();
                paramVals = new Object[paramsInfo.length];
            
                for(int i=0;i<paramsInfo.length;i++)
                {
                    String paramName = paramsInfo[i].getParamName();
                    List elements = rootElem.getChildren(paramName, Namespace.getNamespace(ssNs));
                    if(paramsInfo[i].isParamUserInfo())
                    {
                        paramVals[i] = userInfo;
                    }
                    else if(paramsInfo[i].isParamMXTransaction())
                    {
                        paramVals[i] = mxTrans;
                    }
                    else if(paramsInfo[i].isParamByteArray())
                    {
                        byte[] clearTextData = null;
                        if(elements == null || elements.size()==0)
                        {
                            clearTextData = null;
                        }
                        else
                        {
                            String base64EncData = ((Element)elements.get(0)).getText();
                            clearTextData = javax.xml.bind.DatatypeConverter.parseBase64Binary(base64EncData);
                        }
                        paramVals[i] = clearTextData;
                    }
                    else if(paramsInfo[i].isParamArray())
                    {
                        /*if(elements == null || elements.size()==0)
                        {
                            paramVals[i] = null;
                        }*/
                        //Element elem = rootElem.getChild(paramsInfo[i].getParamName(), Namespace.getNamespace(MosConstants.MAXIMO_NS));
                        if(paramsInfo[i].isParamMbo())
                        {
                            MboSetRemote mboSet = processRequestForMboSet(paramsInfo[i], (Element)elements.get(0), userInfo, mxTrans);
                            paramVals[i] = getMboSetAsArray(mboSet);
                            //mboSet.setMXTransaction(mxTrans);
                        }
                        else//simple java type
                        {
                            paramVals[i] = elementsToArray(elements,paramsInfo[i]);
                        }
                    }
                    else if(paramsInfo[i].isParamList())
                    {
                        if(elements == null || elements.size()==0)
                        {
                            paramVals[i] = null;
                        }
                        else if(paramsInfo[i].isParamMbo())
                        {
                            MboSetRemote mboSet = processRequestForMboSet(paramsInfo[i], (Element)elements.get(0), userInfo, mxTrans);
                            //mboSet.setMXTransaction(mxTrans);
                            paramVals[i] = getMboSetAsList(mboSet);
                        }
                        else//simple java type
                        {
                            paramVals[i] = elementsToList(elements,paramsInfo[i]);
                        }
                    }
                    else if(paramsInfo[i].isParamMbo())
                    {
                        if(elements == null || elements.size()==0)
                        {
                            paramVals[i] = null;
                        }
                        MboQbeXMLProcessor mboQbe = new MboQbeXMLProcessor(paramsInfo[i].getMboName(), userInfo);
                        MboRemote mbo = mboQbe.getMbo((Element)elements.get(0),true);
                        
                        // IJ26653: If the mbo is null, it does not exist in the db. Throw a more useful "doesntexist" error
                        // before a NPE is thrown by the next line.
                        if(mbo == null) {
                        	String[] errParams = {paramsInfo[i].getMboName()};
                        	throw new MXApplicationException("iface", "doesntexist", errParams);
                        }
                        
                        mbo.getThisMboSet().setMXTransaction(mxTrans);
                        paramVals[i] = mbo;
                    }
                    else if(paramsInfo[i].isParamMboSet())
                    {
                        if(elements == null || elements.size()==0)
                        {
                            paramVals[i] = null;
                        }
                        paramVals[i] = processRequestForMboSet(paramsInfo[i], (Element)elements.get(0), userInfo, mxTrans);
                    }
                    else//java simple types
                    {
                        Element simpleElem = rootElem.getChild(paramsInfo[i].getParamName(), Namespace.getNamespace(ssNs));
                        paramVals[i] = deserializeSimpleType(simpleElem, paramsInfo[i].getParamType());
                    }
                }
            }
            boolean hasPriv = maxService.checkSecurity(serviceImplMethod.getName(), userInfo);
            if(!hasPriv)
            {
                String[] params = {serviceImplMethod.getName(),maxService.getName(),userInfo.getLoginID()};
                throw new MXApplicationException("iface","ssmethodnopriv",params);
            }
            Object response = serviceImplMethod.invoke(maxService, paramVals);
            //System.out.println("-response------------"+response);
            byte[] responseData = null;
            if(!methodInfo.isVoidReturnType())
            {
                responseData = processResponse(response, methodInfo, schemaGen.getOpResponseElementName(globalMethodName), userInfo);
                responseData = this.transformFromMaximoBidiFormatTo(responseData, bidiFormat);
            }
            else
            {
            	responseData = StAXUtil.createEmptyResponse(globalMethodName + MosConstants.MESSAGETYPE_RESPONSE, userInfo);
            }
            //XMLUtils.writeBytesToXMLFile(responseData, "c:\\harrier\\poget.xml");
            //System.out.println("-------------"+(new String(responseData)));
            if(mxTrans != null)
            {
                mxTrans.commit();
            }
            return responseData;
        }
        catch(Exception e)
        {
            //MicUtil.INTEGRATIONLOGGER.error(e.getMessage(),e);
            try
            {
                if(mxTrans != null)
                {
                    mxTrans.rollback();
                }
            }
            catch(Exception re)
            {
                //throw new MXApplicationException("system","remote",re);
                MicUtil.INTEGRATIONLOGGER.error(re.getMessage(),re);
            }
            
            Exception actualExc = e;
            
            if(e instanceof InvocationTargetException)
            {
                Throwable t = e.getCause();
                if(t != null)
                {
                    if(t instanceof Exception)
                    {
                        actualExc = (Exception)t;
                    }
                    else 
                    {
                        MicUtil.INTEGRATIONLOGGER.error(t.getMessage(),t);
                    }
                }
            }
            else if(e instanceof RuntimeException)
            {
                throw (RuntimeException)e;
            }
            else if(e instanceof MXException)
            {
                throw (MXException)e;
            }
            else if(actualExc instanceof MXException)
            {
                throw (MXException)actualExc;
            }

            throw new EJBException(actualExc);
        }    
        finally
        {
        	freeDBConnection(userInfo.getConnectionKey());
            MaxLoggerContext.destroyCurrentContext();
        }
    }
    
    
    private byte[] processResponse(Object response, MethodInfo methodInfo, String responseElemName, UserInfo userInfo) 
        throws MXException, RemoteException
    {
        ByteArrayOutputStream bo = null;
        try
        {
            XMLOutputFactory outputFactory = XMLOutputFactory.newInstance();
            bo = new ByteArrayOutputStream();
            XMLStreamWriter writer = outputFactory.createXMLStreamWriter(bo);
            StAXUtil.createHeaderElements(writer, responseElemName);
            StAXUtil.setCommonContentAttributes(writer, userInfo);
            if(response != null)
            {
                serializeReturnValue(writer,methodInfo,response);
            }
            writer.writeEndElement();
            writer.writeEndDocument();
            writer.close();
        }
        catch(XMLStreamException xmlst)
        {
            throw new MXSystemException("iface","xmlstrm", xmlst);
        }
        return bo.toByteArray();
    }
    
    private Object elementsToArray(List elements, ParamInfo paramInfo) throws MXException
    {
        Object array = null;
        List elemValList = elementsToList(elements, paramInfo);
        Class paramCompClazz = paramInfo.getParamCompType();
        if(paramCompClazz.isPrimitive())
        {
            array = createPrimitiveArray(paramCompClazz, elemValList);
        }
        else
        {
            array = Array.newInstance(paramCompClazz, elemValList.size());
            array = elemValList.toArray((Object[])array);
        }
        //createPrimitiveArray(paramsInfo[i].getParamCompType(),);
        return array;
    }
    
    private List<Object> elementsToList(List elements, ParamInfo paramInfo) throws MXException
    {
        List<Object> list = new ArrayList<Object>(elements.size());
        for(int i=0;i<elements.size();i++)
        {
            Element elem = (Element)elements.get(i);
            list.add(deserializeSimpleType(elem, paramInfo.getParamCompType()));
        }
        return list;
    }

    
    private MboSetRemote processRequestForMboSet(ParamInfo paramInfo, 
            Element mboSetFetchElem, UserInfo userInfo, MXTransaction mxTrans)
        throws MXException, RemoteException
    {
        if(mboSetFetchElem==null)
        {
            return null;
        }
        if(paramInfo.getOwnerName() != null)
        {
            MboQbeXMLProcessor mboQbe = new MboQbeXMLProcessor(paramInfo.getOwnerName(), userInfo);
            return mboQbe.getMboSetFromOwner(mboSetFetchElem,paramInfo.getRelation());
        }
        MboQbeXMLProcessor mboQbe = new MboQbeXMLProcessor(paramInfo.getMboName(), userInfo);
        MboSetRemote mboSet = mboQbe.getMboSet(mboSetFetchElem);
        mboSet.setMXTransaction(mxTrans);
        return mboSet;
    }
    
    private MboRemote[] getMboSetAsArray(MboSetRemote mboSet) throws MXException, RemoteException
    {
        List<MboRemote> mboList = getMboSetAsList(mboSet);
        return mboList==null?null:(MboRemote[])mboList.toArray();
    }
    
    private List<MboRemote> getMboSetAsList(MboSetRemote mboSet) throws MXException, RemoteException
    {
        if(mboSet == null) return null;
        MboRemote mboRemote = mboSet.moveFirst();
        List<MboRemote> mboList = new ArrayList<MboRemote>(mboSet.count());
        while(mboRemote != null)
        {
            mboList.add(mboRemote);
            mboRemote = mboSet.moveNext();
        }
        return mboList;
    }

    
    private static Object deserializeSimpleType(Element child, Class parameterType)
        throws MXException
    {
        Object returnObj = null;
        if(child==null)
        {
            if(parameterType.isPrimitive())
            {
                throw new MXApplicationException(
                    "iface", "nullprimitive");
            }
            return null;
        }
        String childName = child.getName();
        
        //check for xsi:nil                        
        if(SchemaUtil.isNil(child))
        {
            if(parameterType.isPrimitive())
            {
                throw new MXApplicationException(
                    "iface", "invalid_nil_attribute", new String[]{childName});
            }
            returnObj = null;                                            
        }
        else
        {
            returnObj = SimpleJavaTypeMapper.createObject(
                parameterType, child.getText());
        }
        return returnObj;
    }

    
    private Object createPrimitiveArray(Class primitiveClass, List objList) throws MXException
    {
        int size = objList.size();
        Object returnArray = Array.newInstance(primitiveClass, size);
        if (int.class.equals(primitiveClass)) 
        {
            for (int i = 0; i < size; i++) 
            {
                Array.setInt(returnArray, i, ((Integer)objList.get(i)).intValue());
            }
        } 
        else if (float.class.equals(primitiveClass)) 
        {
            for (int i = 0; i < size; i++) 
            {
                Array.setFloat(returnArray, i, ((Float)objList.get(i)).floatValue());
            }
        } 
        else if (short.class.equals(primitiveClass)) 
        {
            for (int i = 0; i < size; i++) 
            {
                Array.setShort(returnArray, i, ((Short)objList.get(i)).shortValue());
            }
        } 
        else if (long.class.equals(primitiveClass)) 
        {
            for (int i = 0; i < size; i++) 
            {
                Array.setLong(returnArray, i, ((Long)objList.get(i)).longValue());
            }
        } 
        else if (boolean.class.equals(primitiveClass)) 
        {
            for (int i = 0; i < size; i++) 
            {
                Array.setBoolean(returnArray, i, ((Boolean)objList.get(i)).booleanValue());
            }
        } 
        else if (byte.class.equals(primitiveClass)) 
        {
            for (int i = 0; i < size; i++) 
            {
                Array.setByte(returnArray, i, ((Byte)objList.get(i)).byteValue());
            }
        } 
        else if (double.class.equals(primitiveClass)) 
        {
            for (int i = 0; i < size; i++) 
            {
                Array.setDouble(returnArray, i, ((Double)objList.get(i)).doubleValue());
            }
        } 
        else 
        {
            //objList.toArray((Object[])returnArray);
            throw new MXApplicationException("iface", "unsupported_primitive_array",new String[]{primitiveClass.getName()});
        }
        return returnArray;
    }
    
    
    private void serializeReturnValue(XMLStreamWriter writer,
            MethodInfo methodInfo, Object returnObj) throws MXException
    {
        if(returnObj == null)
            return;
        //System.out.println(methodInfo+" wwww serializing mbo to mos ==="+returnObj);
        try
        {
            //OMElement[] returnElements = null;
            
            ParamInfo returnTypeInfo = methodInfo.getReturnTypeInfo();
            String returnElementName = returnTypeInfo.getParamName();
            Class returnType = returnObj.getClass();
            //OMFactory factory = OMAbstractFactory.getOMFactory();
            //OMNamespace ns1 = factory.createOMNamespace(namespaceURI,"ns");
            
            //check for special case of byte array
            if(SimpleJavaTypeMapper.isByteArrayType(returnObj.getClass()))
            {
                //returnElements = new OMElement[1];
                //OMElement responseElement = factory.createOMElement(returnElementName,ns1);
                //returnElements[0] = responseElement;
                String val = SimpleJavaTypeMapper.getStringValue(returnObj);
                createElement(returnElementName, val, writer);
                return;
            }
            //boolean isArrayOrList = returnTypeInfo.isParamArray()||returnTypeInfo.isParamList();
            if(returnTypeInfo.isParamArray())
            { 
                returnType = returnTypeInfo.getParamCompType();
                Object[] retArray = (Object[])returnObj;
                if(retArray == null || retArray.length == 0)
                    return;
                
                //returnElements = new OMElement[retArray.length];
                if(SimpleJavaTypeMapper.isSimpleJavaType(returnType))
                {
                    for(int i =0; i < retArray.length; i++)
                    {
                        String val = SimpleJavaTypeMapper.getStringValue(retArray[i]);
                        createElement(returnElementName, val, writer);
                    }                
                }
                else if(returnTypeInfo.isParamMbo())
                {
                    StAXStructure stxStruct = null;
                    if(returnTypeInfo.getMosName() == null)
                    {
                        stxStruct = new MBOStAXStructure(false, true);
                    }
                    else
                    {
                        stxStruct = new MOSStAXStructure(null,returnTypeInfo.getMosName(),true,false);
                    }
                    stxStruct.serializeMboArray(writer, (MboRemote[])retArray);
                }
                else if(returnTypeInfo.isParamMboSet())
                {
                    //array of MboSet not supported
                    throw new MXApplicationException(
                        "iface", "Unsupported_ClassType", new String[]{returnType.getName()});
                }
                else
                {
                    //TODO: add support for JavaBean
                    throw new MXApplicationException(
                        "iface", "Unsupported_ClassType", new String[]{returnType.getName()});
                }
            }
            else if(returnTypeInfo.isParamList())
            {
                returnType = returnTypeInfo.getParamCompType();
                List retList = (List)returnObj;
                if(retList==null || retList.size() == 0)
                    return;
                //System.out.println("returnType===="+returnType);
                //returnElements = new OMElement[retArray.length];
                if(SimpleJavaTypeMapper.isSimpleJavaType(returnType))
                {
                    for(int i =0; i < retList.size(); i++)
                    {
                        String val = SimpleJavaTypeMapper.getStringValue(retList.get(i));
                        //System.out.println(i+"val===="+val);
                        //returnElements[i]= factory.createOMElement(returnElementName,ns1);
                        //returnElements[i].setText(val);
                        createElement(returnElementName, val, writer);
                    }                
                }
                else if(returnTypeInfo.isParamMbo())
                {
                    StAXStructure stxStruct = null;
                    if(returnTypeInfo.getMosName() == null)
                    {
                        stxStruct = new MBOStAXStructure(false, true);
                    }
                    else
                    {
                        stxStruct = new MOSStAXStructure(null,returnTypeInfo.getMosName(),true,false);
                    }
                    stxStruct.serializeMboList(writer, retList);
                }
                else if(returnTypeInfo.isParamMboSet())
                {
                    //array of MboSet not supported
                    throw new MXApplicationException(
                        "iface", "Unsupported_ClassType", new String[]{returnType.getName()});
                }
                else
                {
                    //TODO: add support for JavaBean
                    throw new MXApplicationException(
                        "iface", "Unsupported_ClassType", new String[]{returnType.getName()});
                }
            }
            else
            {   
                if(SimpleJavaTypeMapper.isSimpleJavaType(returnType))
                {
                    String val = SimpleJavaTypeMapper.getStringValue(returnObj);
                    createElement(returnElementName, val, writer);
                }
                else if(returnTypeInfo.isParamMbo())
                {
                    MboRemote mbo = (MboRemote)returnObj;
                    StAXStructure stxStruct = null;
                    if(returnTypeInfo.getMosName() == null)
                    {
                        stxStruct = new MBOStAXStructure(false, false);
                    }
                    else
                    {
                        //System.out.println("serializing mbo to mos ==="+returnTypeInfo.getMosName());
                        stxStruct = new MOSStAXStructure(null,returnTypeInfo.getMosName(),false,false);
                    }
                    stxStruct.serializeMbo(writer, mbo);
                }
                else if(returnTypeInfo.isParamMboSet())
                {
                    MboSetRemote mboSet = (MboSetRemote)returnObj;
                    StAXStructure stxStruct = null;
                    if(returnTypeInfo.getMosName() == null)
                    {
                        stxStruct = new MBOStAXStructure();
                    }
                    else
                    {
                        stxStruct = new MOSStAXStructure(null,returnTypeInfo.getMosName(),false,false);
                    }
                    stxStruct.serializeMboSet(writer, mboSet);
                }
                else 
                {
                    //TODO: JavaBean
                    throw new MXApplicationException(
                        "iface", "Unsupported_ClassType", new String[]{returnType.getName()});
                }
            }
        }
        catch(XMLStreamException xmlste)
        {
            throw new MXSystemException("iface","xmlstrm", xmlste);
        }
        catch(RemoteException rme)
        {
            throw new MXSystemException("system","remoteexception", rme);
        }

    }

    
    private void createElement(String elemName, String val, XMLStreamWriter writer) 
        throws XMLStreamException
    {
        writer.writeStartElement(elemName);
        writer.writeCharacters(val);
        writer.writeEndElement();
    }

	private void setUserToContext(UserInfo userInfo)
	{
		IntegrationContext ctx = IntegrationContext.getCurrentContext();
		if(ctx != null && userInfo != null)
		{
			ctx.setProperty("userid", userInfo.getLoginID());
		}
	}

}
