/*
 *
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * 5724-U18
 *
 * (C) COPYRIGHT IBM CORP. 2006,2014
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *
 */




package psdi.iface.util;

import javax.ejb.*;
import javax.naming.*;
import javax.rmi.*;
import javax.servlet.ServletConfig;

import com.ibm.tivoli.maximo.intservice.EnterpriseService;
import com.ibm.tivoli.maximo.intservice.MOSService;
import com.ibm.tivoli.maximo.intservice.WFIntService;
import com.ibm.tivoli.maximo.intservice.WebMethodService;

import java.rmi.*;
import java.util.*;

import psdi.iface.mic.*;
import psdi.iface.mos.*;
import psdi.iface.action.*;
import psdi.iface.gateway.*;
import psdi.util.*;
import psdi.util.logging.MXLogger;
import psdi.webclient.system.websession.WebAppEnv;
import psdi.server.*;
import psdi.iface.router.*;
import psdi.iface.webservices.WebServicesUtil;
import psdi.iface.workflow.WorkFlowServiceLocal;
import psdi.iface.workflow.WorkFlowServiceRemote;

/**
 * @author Anamitra
 *
 */
public class WebUtil
{
    /* Integration Logger */
    private static final MXLogger integrationLogger = MicUtil.INTEGRATIONLOGGER;

    public static final String ENV_USEAPPSERVERSECURITYFORWS = "useAppServerSecurityForWS";

    public static final String ES_CONTEXT = "es";
    public static final String ESQUEUE_CONTEXT = "esqueue";
    public static final String OS_CONTEXT = "os";
    public static final String SS_CONTEXT = "ss";
    public static final String WF_CONTEXT = "wf";


    public static final String WS_SERVLET_NAME = "AxisServlet";
    public static final String WS_AUTH_URL = "authurl";

    private static boolean isStartedInProcess()
    {
        return MXServerInfo.getMXServerInfo().isStartedInProcess();
        //return false;
    }

    private static String getAuthUrl(ServletConfig servletConfig)
    {
        /*String useAppServerSecurityForWS = WebAppEnv.getEnvEntryValue(ENV_USEAPPSERVERSECURITYFORWS);
        boolean val = true;
        if (useAppServerSecurityForWS != null)
        {
            if (useAppServerSecurityForWS.equalsIgnoreCase("false"))
            {
                val = false;
            }
        }
        return val;*/
        String authUrl = servletConfig.getInitParameter(WS_AUTH_URL);
        if(authUrl != null && !authUrl.endsWith("/"))
        {
            authUrl += "/";
        }
        return authUrl;
    }

    private static boolean isWSCall(ServletConfig servletConfig)
    {
        return servletConfig.getServletName().equals(WS_SERVLET_NAME);
    }

    /**
     * Load the EJB handle
     *
     * @throws RemoteException
     *             Remote Exception
     * @throws MXException
     *             Maximo Exception
     * @throws CreateException
     *             Create Exception
     */
    public static Object loadEJBHandle(Class homeClass, String ejbRemoteRefName, String ejbLocalRefName)
        throws MXException
    {
        InitialContext context = null;
        try
        {
            context = new InitialContext();
            return loadEJBHandle(context, homeClass, ejbRemoteRefName, ejbLocalRefName);
        }
        catch (NamingException name)
        {
            if (integrationLogger.isErrorEnabled())
            {
                integrationLogger.error("Naming Exception", name);
            }
            throw new MXSystemException("iface", "jnditreesetuperr", name);
        }
    }

    /**
     * Load the EJB handle
     *
     * @throws RemoteException
     *             Remote Exception
     * @throws MXException
     *             Maximo Exception
     * @throws CreateException
     *             Create Exception
     */
    public static Object loadEJBHandle(InitialContext context, Class homeClass, String ejbRemoteRefName, String ejbLocalRefName)
        throws MXException
    {
        Object homeObj = null;
        String ejbRefName = null;
        try
        {
            if(isStartedInProcess())
            {
                ejbRefName = ejbLocalRefName;
            }
            else
            {
                ejbRefName = ejbRemoteRefName;
            }
            // lookup the Home interface of the ejb
            homeObj = context.lookup("java:comp/env/"
                    + ejbRefName);
            // homeInterface = context.lookup("syncMaximoDataService");
        }
        catch (NamingException name)
        {
            if (integrationLogger.isErrorEnabled())
            {
                integrationLogger.error("Naming Exception", name);
            }
            throw new MXSystemException("iface", "jnditreesetuperr", name);
        }
        finally
        {
            if (context != null)
            {
                try
                {
                    context.close();
                }
                catch (NamingException ignore)
                {
                }
            }
        }
        try
        {
            if(!isStartedInProcess())
            {
                homeObj = PortableRemoteObject.narrow(
                    homeObj, homeClass);
            }
            java.lang.reflect.Method createMethod = homeObj.getClass().getMethod("create", (Class[])null);
            return createMethod.invoke(homeObj, (Object[])null);
        }
        catch(Exception e)
        {
            throw new MXApplicationException("iface","loadejb",e);
        }
    }


    /**
     * Invoke the EJB and return the response
     *
     * @param loginid the login id
     * @param password 
     * @param ejbObject ejb handle
     * @throws RemoteException
     *             Remote Exception
     * @throws MXException
     *             Maximo Exception
     */
    public static byte[] invokeEnterpriseEJB(String loginid, String password, Object ejbObject,
            String extsysName, String entrService, byte[] xmlData, boolean synch, ServletConfig servletConfig) throws RemoteException, MXException
    {
        byte[] response = null;
        if (xmlData == null || xmlData.length == 0) // sanity check
        {
            throw new MXApplicationException("iface", "servlet_no_xmldata");
        }
        if (MXServer.getMXServer().isAdminModeOn(false))
        {
			throw new MXApplicationException("system", "NoLogin", new Object[] {loginid});
        }
        boolean wsCall = isWSCall(servletConfig);
        String authUrl = getAuthUrl(servletConfig);
        boolean allowDfltLogin = SecurityUtil.isAllowDefaultLogin();
        if (synch)
        {
            if(WebAppEnv.useAppServerSecurity())
            {
            	if(loginid != null && password != null && loginid.equals(SecurityUtil.APIKEYLOGIN))
            	{
            		String apikey = password;
            		EnterpriseService service = new EnterpriseService();
            		response = service.maxAPIKeySecureProcessExternalDataSync(apikey, xmlData,
                            entrService, extsysName);
            	}
            	else if(!wsCall || (wsCall && authUrl == null))
                {
            		if(allowDfltLogin)
            		{
                		EnterpriseService service = new EnterpriseService();
                		response = service.processExternalDataSync(xmlData, entrService, extsysName);
            		}
            		else
            		{
	                    if(isStartedInProcess())
	                    {
	                        response = ((MEAGatewayLocal)ejbObject).secureProcessExternalDataSync(xmlData,
	                            entrService, extsysName);
	                    }
	                    else
	                    {
	                        response = ((MEAGateway)ejbObject).secureProcessExternalDataSync(xmlData,
	                            entrService, extsysName);
	                    }
            		}
                }
                else
                {
                    if(loginid != null && password != null && authUrl != null)
                    {
                        //redirect the call to the action servlet with HTTP basic auth header
                        HTTPHandler handler = new HTTPHandler();
                        Map<String,String> map = new HashMap<String,String>();
                        map.put(HTTPHandler.USERNAME, loginid);
                        map.put(HTTPHandler.PASSWORD, password);
                        String asUrl = WebServicesUtil.getResourceURL(authUrl, ES_CONTEXT+"/"
                            +extsysName+"/"+ entrService);
                        map.put(HTTPHandler.URL, asUrl);
                        handler.invoke(map, xmlData);
                    }
                    else//Is it an error [as no auth info is provided] - have to discuss?
                    {
                        /*if(isStartedInProcess())
                        {
                            response = ((MEAGatewayLocal)ejbObject).processExternalDataSync(xmlData,
                                entrService, extsysName);
                        }
                        else
                        {
                            response = ((MEAGateway)ejbObject).processExternalDataSync(xmlData,
                                entrService, extsysName);
                        }*/
                        throw new MXApplicationException("iface","noauthinfo");
                    }
                }
            }
            else
            {
                if(loginid != null && password != null)
                {
                	if(loginid.equals(SecurityUtil.APIKEYLOGIN))
                	{
                		String apikey = password;
                		EnterpriseService service = new EnterpriseService();
                		response = service.maxAPIKeySecureProcessExternalDataSync(apikey, xmlData,
	                            entrService, extsysName);
                	}
                	else
                	{
	                    if(isStartedInProcess())
	                    {
	                        response = ((MEAGatewayLocal)ejbObject).maxSecureProcessExternalDataSync(loginid,password,xmlData,
	                            entrService, extsysName);
	                    }
	                    else
	                    {
	                        response = ((MEAGateway)ejbObject).maxSecureProcessExternalDataSync(loginid,password,xmlData,
	                            entrService, extsysName);
	                    }
                	}
                }
                else
                {
                    if(isStartedInProcess())
                    {
                        response = ((MEAGatewayLocal)ejbObject).processExternalDataSync(xmlData,
                            entrService, extsysName);
                    }
                    else
                    {
                        response = ((MEAGateway)ejbObject).processExternalDataSync(xmlData,
                            entrService, extsysName);
                    }
                }
            }
        }
        else
        {

            if(WebAppEnv.useAppServerSecurity())
            {
            	if(loginid != null && password != null && loginid.equals(SecurityUtil.APIKEYLOGIN))
            	{
            		String apikey = password;
            		EnterpriseService service = new EnterpriseService();
            		response = service.maxAPIKeySecureProcessExternalDataAsync(apikey, xmlData,
                            entrService, extsysName);
            	}
            	else if(!wsCall || (wsCall && authUrl == null))
                {
                    if(isStartedInProcess())
                    {
                    	response = ((MEAGatewayLocal)ejbObject).secureProcessExternalDataAsync(xmlData,
                            entrService, extsysName);
                    }
                    else
                    {
                    	response = ((MEAGateway)ejbObject).secureProcessExternalDataAsync(xmlData,
                            entrService, extsysName);
                    }
                }
                else
                {
                    if(loginid != null && password != null && authUrl != null)
                    {
                        //redirect the call to the action servlet with HTTP basic auth header
                        HTTPHandler handler = new HTTPHandler();
                        Map<String,String> map = new HashMap<String,String>();
                        map.put(HTTPHandler.USERNAME, loginid);
                        map.put(HTTPHandler.PASSWORD, password);
                        String asUrl = WebServicesUtil.getResourceURL(authUrl, ESQUEUE_CONTEXT+"/"
                            +extsysName+"/"+ entrService);
                        map.put(HTTPHandler.URL, asUrl);
                        response = handler.invoke(map, xmlData);
                    }
                    else//Is it an error [as no auth info is provided] - have to discuss?
                    {
                        /*if(isStartedInProcess())
                        {
                        	response = ((MEAGatewayLocal)ejbObject).processExternalDataAsync(xmlData,
                                entrService, extsysName);
                        }
                        else
                        {
                        	response = ((MEAGateway)ejbObject).processExternalDataAsync(xmlData,
                                entrService, extsysName);
                        }*/
                        throw new MXApplicationException("iface","noauthinfo");
                    }
                }
            }
            else if(loginid != null && password != null)
            {
            	if(loginid.equals(SecurityUtil.APIKEYLOGIN))
            	{
            		String apikey = password;
            		EnterpriseService service = new EnterpriseService();
            		response = service.maxAPIKeySecureProcessExternalDataAsync(apikey, xmlData,
                            entrService, extsysName);
            	}
            	else
            	{
	                if(isStartedInProcess())
	                {
	                	response = ((MEAGatewayLocal)ejbObject).maxSecureProcessExternalDataAsync(loginid,password,xmlData,
	                        entrService, extsysName);
	                }
	                else
	                {
	                	response = ((MEAGateway)ejbObject).maxSecureProcessExternalDataAsync(loginid,password,xmlData,
	                        entrService, extsysName);
	                }
            	}
            }
            else
            {
                if(isStartedInProcess())
                {
                	response = ((MEAGatewayLocal)ejbObject).processExternalDataAsync(xmlData, entrService,
                        extsysName);
                }
                else
                {
                	response = ((MEAGateway)ejbObject).processExternalDataAsync(xmlData, entrService,
                        extsysName);
                }
            }
        }

        return response;
    }

    /**
     * Invoke the EJB and return the response
     *
     * @throws RemoteException
     *             Remote Exception
     * @throws MXException
     *             Maximo Exception
     */
    public static byte[] invokeMosEJB(String loginid, String password, Object ejbObject,
            String mosName, byte[] xmlData, ServletConfig servletConfig) throws RemoteException, MXException
    {
        byte[] response = null;
        if (xmlData == null || xmlData.length == 0) // sanity check
        {
            throw new MXApplicationException("iface", "servlet_no_xmldata");
        }
//        System.out.println("admin mode 11- "+MXServer.getMXServer().isAdminModeOn(false));
//        System.out.println("admin mode 22- "+MXServer.getMXServer().isAdminModeOn(true));

        if (MXServer.getMXServer().isAdminModeOn(false))
        {
			throw new MXApplicationException("system", "NoLogin", new Object[] {loginid});
        }
        boolean wsCall = isWSCall(servletConfig);
        String authUrl = getAuthUrl(servletConfig);
        if(WebAppEnv.useAppServerSecurity())
        {
        	if(loginid != null && password != null && loginid.equals(SecurityUtil.APIKEYLOGIN))
        	{
        		String apikey = password;
        		MOSService service = new MOSService();
        		response = service.maxAPIKeySecureProcessMOS(apikey, xmlData, mosName);
        	}
        	else if(!wsCall || (wsCall  && authUrl == null))
            {
                if(isStartedInProcess())
                {
                    response = ((MOSServiceLocal)ejbObject).secureProcessMOS(xmlData,
                        mosName);
                }
                else
                {
                    response = ((MOSServiceRemote)ejbObject).secureProcessMOS(xmlData,
                        mosName);
                }
            }
            else
            {
                if(loginid != null && password != null && authUrl != null)
                {
                    //redirect the call to the action servlet with HTTP basic auth header
                    HTTPHandler handler = new HTTPHandler();
                    Map<String,String> map = new HashMap<String,String>();
                    map.put(HTTPHandler.USERNAME, loginid);
                    map.put(HTTPHandler.PASSWORD, password);
                    String asUrl = WebServicesUtil.getResourceURL(authUrl, OS_CONTEXT+"/"
                        + mosName);
                    map.put(HTTPHandler.URL, asUrl);
                    handler.invoke(map, xmlData);
                }
                else//Is it an error [as no auth info is provided] - have to discuss?
                {
                    /*if(isStartedInProcess())
                    {
                        response = ((MOSServiceLocal)ejbObject).processMOS(xmlData,
                            mosName);
                    }
                    else
                    {
                        response = ((MOSServiceRemote)ejbObject).processMOS(xmlData,
                            mosName);
                    }*/
                    throw new MXApplicationException("iface","noauthinfo");
                }

            }
        }
        else if(loginid != null && password != null)
        {
        	if(loginid.equals(SecurityUtil.APIKEYLOGIN))
        	{
        		String apikey = password;
        		MOSService service = new MOSService();
        		response = service.maxAPIKeySecureProcessMOS(apikey, xmlData, mosName);
        	}
        	else
        	{
	            if(isStartedInProcess())
	            {
	                response = ((MOSServiceLocal)ejbObject).maxSecureProcessMOS(loginid,password,xmlData,
	                    mosName);
	            }
	            else
	            {
	                response = ((MOSServiceRemote)ejbObject).maxSecureProcessMOS(loginid,password,xmlData,
	                    mosName);
	            }
        	}
        }
        else
        {
            if(isStartedInProcess())
            {
                response = ((MOSServiceLocal)ejbObject).processMOS(xmlData,
                    mosName);
            }
            else
            {
                response = ((MOSServiceRemote)ejbObject).processMOS(xmlData,
                    mosName);
            }
        }
        return response;
    }

    /**
     * Invoke the EJB and return the response
     *
     * @throws RemoteException
     *             Remote Exception
     * @throws MXException
     *             Maximo Exception
     */
    public static byte[] invokeActionEJB(String loginid, String password, Object ejbObject,
            String appServiceName, byte[] xmlData, ServletConfig servletConfig) throws RemoteException, MXException
    {

        //ejbRemote = loadEJBHandle();
        byte[] response = null;
        if (xmlData == null || xmlData.length == 0) // sanity check
        {
            throw new MXApplicationException("iface", "servlet_no_xmldata");
        }
        if (MXServer.getMXServer().isAdminModeOn(false))
        {
			throw new MXApplicationException("system", "NoLogin", new Object[] {loginid});
        }
        boolean wsCall = isWSCall(servletConfig);
        String authUrl = getAuthUrl(servletConfig);
        if(WebAppEnv.useAppServerSecurity())
        {
        	if(loginid != null && password != null && loginid.equals(SecurityUtil.APIKEYLOGIN))
        	{
        		String apikey = password;
        		WebMethodService service = new WebMethodService();
        		response = service.maxAPIKeySecureAction(apikey, xmlData,
                        appServiceName);
        	}
        	else if(!wsCall || (wsCall && authUrl == null))
            {
                if(isStartedInProcess())
                {
                    response = ((MAXActionServiceLocal)ejbObject).secureAction(xmlData,
                        appServiceName);
                }
                else
                {
                    response = ((MAXActionServiceRemote)ejbObject).secureAction(xmlData,
                        appServiceName);
                }
            }
            else
            {
                if(loginid != null && password != null && authUrl != null)
                {
                    //redirect the call to the action servlet with HTTP basic auth header
                    HTTPHandler handler = new HTTPHandler();
                    Map<String,String> map = new HashMap<String,String>();
                    map.put(HTTPHandler.USERNAME, loginid);
                    map.put(HTTPHandler.PASSWORD, password);
                    String asUrl = WebServicesUtil.getResourceURL(WebServicesUtil.getMEAWebAPPURL(), "/"+SS_CONTEXT+"/"
                        + appServiceName);
                    map.put(HTTPHandler.URL, asUrl);
                    handler.invoke(map, xmlData);
                }
                else//Is it an error [as no auth info is provided] - have to discuss?
                {
                    /*if(isStartedInProcess())
                    {
                        response = ((MAXActionServiceLocal)ejbObject).action(xmlData,
                            appServiceName);
                    }
                    else
                    {
                        response = ((MAXActionServiceRemote)ejbObject).action(xmlData,
                            appServiceName);
                    }*/
                    throw new MXApplicationException("iface","noauthinfo");
                }
            }
        }
        else
        {
            if(loginid != null && password != null)
            {
                
            	if(loginid.equals(SecurityUtil.APIKEYLOGIN))
            	{
            		String apikey = password;
            		WebMethodService service = new WebMethodService();
            		response = service.maxAPIKeySecureAction(apikey, xmlData,
	                        appServiceName);
            	}
            	else
            	{
            		if(isStartedInProcess())
	                {
	                    response = ((MAXActionServiceLocal)ejbObject).maxSecureAction(loginid,password,xmlData,
	                        appServiceName);
	                }
	                else
	                {
	                    response = ((MAXActionServiceRemote)ejbObject).maxSecureAction(loginid,password,xmlData,
	                        appServiceName);
	                }
            	}
            }
            else
            {
                if(isStartedInProcess())
                {
                    response = ((MAXActionServiceLocal)ejbObject).action(xmlData,
                        appServiceName);
                }
                else
                {
                    response = ((MAXActionServiceRemote)ejbObject).action(xmlData,
                        appServiceName);
                }
            }
        }
        return response;
    }

    /**
     * Invoke the EJB and return the response
     *
     * @throws RemoteException
     *             Remote Exception
     * @throws MXException
     *             Maximo Exception
     */
    public static byte[] invokeWFEJB(String loginid, String password, Object ejbObject,
            String wfName, byte[] xmlData, ServletConfig servletConfig) throws RemoteException, MXException
    {

        //ejbRemote = loadEJBHandle();
        byte[] response = null;
        if (xmlData == null || xmlData.length == 0) // sanity check
        {
            throw new MXApplicationException("iface", "servlet_no_xmldata");
        }
        if (MXServer.getMXServer().isAdminModeOn(false))
        {
			throw new MXApplicationException("system", "NoLogin", new Object[] {loginid});
        }
        if(WebAppEnv.useAppServerSecurity())
        {
           	if(loginid != null && password != null && loginid.equals(SecurityUtil.APIKEYLOGIN))
        	{
        		String apikey = password;
        		WFIntService service = new WFIntService();
        		response = service.maxAPIKeySecureProcessWorkflow(apikey, xmlData,
                        wfName);
        	}
        	else 
        	{
        		if(isStartedInProcess())
	            {
	                response = ((WorkFlowServiceLocal)ejbObject).secureProcessWorkflow(xmlData,
	                    wfName);
	            }
	            else
	            {
	                response = ((WorkFlowServiceRemote)ejbObject).secureProcessWorkflow(xmlData,
	                    wfName);
	            }
        	}
        }
        else
        {
            if(loginid != null && password != null)
            {
            	if(loginid.equals(SecurityUtil.APIKEYLOGIN))
            	{
            		String apikey = password;
            		WFIntService service = new WFIntService();
            		response = service.maxAPIKeySecureProcessWorkflow(apikey, xmlData,
	                        wfName);
            	}
            	else
            	{
	                if(isStartedInProcess())
	                {
	                    response = ((WorkFlowServiceLocal)ejbObject).maxSecureProcessWorkflow(loginid, password, xmlData,
	                        wfName);
	                }
	                else
	                {
	                    response = ((WorkFlowServiceRemote)ejbObject).maxSecureProcessWorkflow(loginid, password,xmlData,
	                        wfName);
	                }
            	}
            }
            else
            {
                if(isStartedInProcess())
                {
                    response = ((WorkFlowServiceLocal)ejbObject).processWorkflow(xmlData,
                        wfName);
                }
                else
                {
                    response = ((WorkFlowServiceRemote)ejbObject).processWorkflow(xmlData,
                        wfName);
                }
            }
        }
        return response;
    }

}
