/*
 *
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * 5724-U18
 *
 * (C) COPYRIGHT IBM CORP. 2011,2014
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *
 */
package psdi.iface.webservices;

import java.rmi.RemoteException;

import com.ibm.tivoli.maximo.intservice.MOSService;

import psdi.iface.mic.MicConstants;
import psdi.iface.mos.MOSServiceLocal;
import psdi.iface.mos.MOSServiceRemote;
import psdi.iface.util.SecurityUtil;
import psdi.util.MXApplicationException;
import psdi.util.MXException;
import psdi.webclient.system.websession.WebAppEnv;

/**
 * Web service proxy for object structure services.
 * @author Anamitra
 *
 */
public class OSWebServiceProxy extends BaseWebServiceProxy 
{

	/* (non-Javadoc)
	 * @see psdi.iface.webservices.BaseWebServiceProxy#getEJBLocalRefName()
	 */
	@Override
	protected String getEJBLocalRefName() 
	{
		return MicConstants.MOS_EJB_LOCAL_REF_KEY;
	}

	/* (non-Javadoc)
	 * @see psdi.iface.webservices.BaseWebServiceProxy#getEJBRemoteRefName()
	 */
	@Override
	protected String getEJBRemoteRefName() 
	{
		return MicConstants.MOS_EJB_REF_KEY;
	}

	/* (non-Javadoc)
	 * @see psdi.iface.webservices.BaseWebServiceProxy#invokeService(java.lang.String, java.lang.String, byte[])
	 */
	@Override
	protected byte[] invokeService(String loginid, String password, byte[] req)
			throws MXException, RemoteException 
	{
        byte[] response = null;
        Object ejbObject = getEJBObject();
        getLogger().debug("MOSWebServiceProvider - Got the EJB handle "+ejbObject);

        if (req == null || req.length == 0) // sanity check
        {
            throw new MXApplicationException("iface", "servlet_no_xmldata");
        }
        
        boolean allowDfltLogin = SecurityUtil.isAllowDefaultLogin();
        
        if(WebAppEnv.useAppServerSecurity())
        {
            getLogger().debug("MOSWebServiceProvider - app server security turned on");
        	if(SecurityUtil.APIKEYLOGIN.equals(loginid))
        	{
        		String apikey = password;
        		MOSService service = new MOSService();
        		response = service.wsMaxAPIKeySecureProcessMOS(apikey, req, serviceName);
        	}
        	else
        	{
        		if(allowDfltLogin)
        		{
            		MOSService service = new MOSService();
            		response = service.processMOS(req, serviceName);
        		}
        		else
        		{
		            if(isStartedInProcess())
		            {
		                response = ((MOSServiceLocal)ejbObject).wsSecureProcessMOS(req,
		                		serviceName);
		            }
		            else
		            {
		                response = ((MOSServiceRemote)ejbObject).wsSecureProcessMOS(req,
		                		serviceName);
		            }
        		}
        	}
        }
        else if(loginid != null && password != null)
        {
        	if(loginid.equals(SecurityUtil.APIKEYLOGIN))
        	{
        		String apikey = password;
        		MOSService service = new MOSService();
        		response = service.wsMaxAPIKeySecureProcessMOS(apikey, req, serviceName);
        	}
        	else
        	{
	            getLogger().debug("MOSWebServiceProvider - using maximo authentication info with loginid "+loginid);
	
	            if(isStartedInProcess())
	            {
	                response = ((MOSServiceLocal)ejbObject).wsMaxSecureProcessMOS(loginid,password,req,
	                		serviceName);
	            }
	            else
	            {
	                response = ((MOSServiceRemote)ejbObject).wsMaxSecureProcessMOS(loginid,password,req,
	                		serviceName);
	            }
        	}
        }
        else
        {
            getLogger().debug("MOSWebServiceProvider - using default authentication");

            if(isStartedInProcess())
            {
                response = ((MOSServiceLocal)ejbObject).wsProcessMOS(req,
                		serviceName);
            }
            else
            {
                response = ((MOSServiceRemote)ejbObject).wsProcessMOS(req,
                		serviceName);
            }
        }
        return response;

	}

}